"use strict";

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var _require = require('./rxjs-adapter'),
    scan = _require.scan,
    startWith = _require.startWith,
    createEventSubject = _require.createEventSubject,
    mergeObservable = _require.mergeObservable;

var _require2 = require('./util'),
    safeWarn = _require2.safeWarn;

var makeInteractions = require('./interactions');

var _require3 = require('react'),
    useState = _require3.useState,
    useEffect = _require3.useEffect;

function defaultMapStateObservable(observable) {
  return scan(function scan(state, mod) {
    if (typeof mod === 'function') {
      return mod(state);
    } else {
      safeWarn('Interaction is not defined as operator (function).');
    }
  })(observable);
}

function useInteractions(initialState, defineInteractions, sinks, mapStateObservable) {
  var interactions = makeInteractions(createEventSubject);
  var interactionList = Array.isArray(sinks) ? sinks : [];
  var names = Object.keys(defineInteractions);

  for (var i = 0; i < names.length; i++) {
    var name = names[i];
    var definitionFn = defineInteractions[name];
    var obs = interactions.get(name);
    interactionList.push(definitionFn.call(defineInteractions, obs));
  }

  var cycle = {
    subscribe: function subscribe() {
      var observable = startWith(initialState)(mergeObservable(interactionList));
      var mappedObs = mapStateObservable ? mapStateObservable(observable) : defaultMapStateObservable(observable);
      var subscription = mappedObs.subscribe(function next(value) {
        cycle.setState(value);
      });
      return function unsubscribe() {
        subscription.unsubscribe();
      };
    },
    // setState: To be replaced in useCycle.
    setState: function setState() {
      safeWarn('setState is called before component is mounted');
    },
    interactions: interactions
  };
  var subscribe = cycle.subscribe.bind(cycle);

  function useCycle() {
    var _useState = useState(initialState),
        _useState2 = _slicedToArray(_useState, 2),
        state = _useState2[0],
        setState = _useState2[1];

    cycle.setState = setState;
    useEffect(subscribe, []);
    return state;
  }

  return [interactions, useCycle];
}

module.exports = {
  useInteractions: useInteractions
};