"use strict";

var _require = require('./util'),
    safeWarn = _require.safeWarn;

function makeInteractions(createEventSubject) {
  var subjects = {};

  function get(name) {
    if (name === null || typeof name !== 'string') {
      throw new Error('Invalid name for the interaction collection.');
    }

    if (!subjects[name]) {
      subjects[name] = createEventSubject();
    }

    return subjects[name];
  }

  function listener(name) {
    var eventSubject = subjects[name];

    if (!eventSubject) {
      safeWarn('Listening event "' + name + '" before using interactions.get("' + name + '")');
    }

    return (eventSubject || get(name)).onEvent;
  }

  function bindListeners(interactionTypes) {
    var result = {};
    var names = Object.keys(interactionTypes);

    for (var i = 0; i < names.length; i++) {
      var name = names[i];
      result[name] = listener(interactionTypes[name]);
    }

    return result;
  }

  function bindAllListeners() {
    var result = {};
    var names = Object.keys(subjects);

    for (var i = 0; i < names.length; i++) {
      var name = names[i];
      result[name] = listener(name);
    }

    return result;
  }

  function _getCurrentListeners() {
    var result = {};
    var names = Object.keys(subjects);

    for (var i = 0; i < names.length; i++) {
      var name = names[i];
      result[name] = listener(name);
    }

    return result;
  }

  listener.get = get;
  listener.listener = listener;
  listener.bindListeners = bindListeners;
  listener.bindAllListeners = bindAllListeners;
  listener._getCurrentListeners = _getCurrentListeners;
  return listener;
}

module.exports = makeInteractions;